/*
 * Programmieren fuer Physiker, SS 2023
 *
 * Beispiel zur Verwendung der CImg Grafik-Bibliothek
 * Compilieren: Linux: Bibliotheken X11, pthread benoetigt (-lX11 -lpthread)
 *              Windows: Bibliothek gdi32 benoetigt (-lgdi32)
 */

#include <iostream>
#include "CImg.h"   // load CImg.h from the current directory
using namespace cimg_library;  
using namespace std ;

void green_dots( CImg<unsigned char>& canvas, int x, int y) 
// example how to pass an image to a function
{
    const unsigned char green[] = { 0,255,0 } ;
    canvas.draw_point( x, y, green) ; // draw into the image
    return ;
}

int main() 
{
    // create an image (to draw on)
    // size 300 x 200, where (0,0) is upper left corner. z-depth is 1. 
    // color channels are 3. default color-value is 0 (all black) 
    CImg<unsigned char> canvas( 300, 200, 1, 3, 0);

    // create a window, connect the canvas to the window, set the title-string
    CImgDisplay window( canvas, "A CImg-Example");

    // colors are arrays with 3 components: red, green and blue intensity
    const unsigned char black[3] = { 0, 0, 0} ,
	                white[3] = { 255, 255, 255},
	                red[3] = { 255, 0,0 }, 
		        green[3] = { 0, 255, 0 }, 
		        blue[3] = { 0, 0, 255 },
			yellow[3] = { 255, 255, 0 };
	
    // fill the whole image with "0" -> all is black
    canvas.fill( 0) ;
		
    // draw two lines, parameters (x1, y1, x2, y2, color, opacity=1.0) 
    canvas.draw_line( 50, 100, 250, 100, white, 1.0) ;
    canvas.draw_line( 150, 0, 150, 199, white) ;

    // filled circle, parameters (xcenter, ycenter, radius, color, opacity=1.0) 
    canvas.draw_circle( 150, 100, 70, red, 0.5) ;
    // outlined circle,
    //   parameters (xcenter, ycenter, radius, color, opacity=1.0, ~0U) 
    canvas.draw_circle( 150, 100, 60, green, 1.0, ~0U) ;

    // text, param: char[], x, y, fg-color, bg-color, opacity=1.0, font-size)
    canvas.draw_text( 10, 10, "CImp is great!", black, yellow, 1.0, 26) ;

    // there are a lot more: draw_rectangle, draw_triangle, draw_ellipse,...
    // there are also a lot of functions for "image-processing"

    // how to pass the canvas to a function? see this example
    green_dots( canvas, 140, 90) ;
    green_dots( canvas, 160, 90) ;

    // show the canvas... (this is to finally display all these drawing commands!)
    window.display(canvas) ; 

    // example how to query the mouse
    cout << "Click into the window of the image..." << endl ;
    while (!window.is_closed()) // window still alive?
    {
	window.wait(); // wait for an event (mouse) in the window ;
	if (window.button() != 0)
	{
	    cout << "Maus-Knopf Nr: " << window.button() << endl ;
	    cout << "  an den Koordinaten (x,y): " << window.mouse_x() 
		 << " " << window.mouse_y() << endl ;
	}
	if (window.key() != 0)
	{
	    cout << "Taste Nr. " << window.key() << endl ;
	    cout << "  mit Maus an den Koordinaten (x,y): " 
		 << window.mouse_x()  << " " << window.mouse_y() << endl ;
	}
    }
    cout << "Bye" << endl ;
}

